package jobqueue

import (
	"github.com/robfig/cron/v3"
	"github.com/stretchr/testify/assert"
	"go.uber.org/zap"
	"os"
	"strings"
	"testing"
	"time"
)

func TestRoundTrip(t *testing.T) {

	if os.Getenv("TEST_BY_TASK") != "" {
		t.Skip("Skipping test because TEST_BY_TASK is set")
	}

	// type JobImport struct {
	//	ID           string          `yaml:"id" json:"id"`
	//	Priority     int             `yaml:"priority" json:"priority"`
	//	Timeout      time.Duration   `yaml:"timeout" json:"timeout"`
	//	MaxRetries   uint            `yaml:"maxRetries" json:"maxRetries"`
	//	RetryDelay   time.Duration   `yaml:"retryDelay" json:"retryDelay"`
	//	Dependencies []string        `yaml:"dependencies" json:"dependencies,omitempty"`
	//	Runnable     RunnableImport  `yaml:"runnable" json:"runnable"`
	//	Scheduler    SchedulerImport `yaml:"scheduler" json:"scheduler,omitempty"`
	//}
	//
	//type RunnableImport struct {
	//	Type string         `yaml:"type" json:"type"`
	//	Data map[string]any `yaml:"data,omitempty" json:"data,omitempty"`
	//}
	//
	//type SchedulerImport struct {
	//	Type     string        `yaml:"type" json:"type"`
	//	Interval time.Duration `yaml:"interval,omitempty" json:"interval,omitempty"`
	//	Spec     string        `yaml:"spec,omitempty" json:"spec,omitempty"`
	//	Delay    time.Duration `yaml:"delay,omitempty" json:"delay,omitempty"`
	//	Event    string        `yaml:"event,omitempty" json:"event,omitempty"`
	//}

	// define test data with jobs in yaml format
	testData := []byte(`
- id: job1
  priority: 1
  timeout: 1s
  maxRetries: 3
  retryDelay: 1s
  runnable:
   type: shell
   data:
     script: echo "~1~ $(date "+%M:%S")" >> /tmp/job1.log
  scheduler:
   type: cron
   spec: "*/10 * * * * *"
- id: job2
  priority: 1
  timeout: 1s
  maxRetries: 3
  retryDelay: 1s
  runnable:
   type: shell
   data:
     script: echo "~~~~2~ $(date "+%M:%S")" >> /tmp/job1.log
  scheduler:
   type: cron
   spec: "*/5 * * * * *"
- id: job3
  priority: 1
  timeout: 1s
  maxRetries: 3
  retryDelay: 1s
  runnable:
    type: shell
    data:
      script: echo "~~~~~~3~ $(date "+%M:%S")" >> /tmp/job1.log
  scheduler:
    type: interval
    interval: 20s
`)

	var err error

	cronInstance := cron.New(cron.WithSeconds())
	cronInstance.Start()

	zapLogger, _ := zap.NewDevelopment()
	_ = zapLogger

	manager := NewManager()
	manager.SetLogger(&ZapAdapter{logger: zapLogger})

	manager.SetCronInstance(cronInstance)
	worker := NewLocalWorker(10)
	worker.SetManager(manager)
	err = manager.AddWorker(worker)

	assert.Nil(t, err)

	err = manager.Start()
	assert.Nil(t, err)

	reader := strings.NewReader(string(testData))

	err = ImportJobsAndSchedule(reader, "yaml", manager)
	assert.Nil(t, err)

	time.Sleep(10 * time.Minute)

}
