package jobqueue

import (
	"github.com/stretchr/testify/assert"
	"math"
	"sync/atomic"
	"testing"
	"time"
)

func TestResourceStats(t *testing.T) {
	stats := NewResourceStats()

	expectedCPUUsage := 75.5
	expectedMemoryUsage := uint64(1024 * 1024) // 1 MB

	cpuBits := math.Float64bits(expectedCPUUsage)
	atomic.StoreUint64(&stats.cpuUsage, cpuBits)
	atomic.StoreUint64(&stats.memoryUsage, expectedMemoryUsage)

	actualCPUUsage := stats.GetCpuUsage()
	actualMemoryUsage := stats.GetMemoryUsage()

	assert.InDelta(t, expectedCPUUsage, actualCPUUsage, 0.001) // Überprüfe auf Genauigkeit bis auf 0,001
	assert.Equal(t, expectedMemoryUsage, actualMemoryUsage)
}

func TestResourceMonitoringStartStop(t *testing.T) {
	stats := NewResourceStats()

	err := stats.StartMonitoring(1 * time.Second)
	assert.Nil(t, err)
	defer stats.StopMonitoring()

	time.Sleep(100 * time.Millisecond)

	assert.NotNil(t, stats.context)
	assert.Nil(t, stats.context.Err())

	stats.StopMonitoring()

	time.Sleep(100 * time.Millisecond)

	assert.Nil(t, stats.context)
}
